from tensorflow.python.keras.layers import Permute

from keras_data_format_converter.layers.confighandlers.dims import handle_transpose_dims


def test_transpose_dims_channels_first_transposable_identity_permutation():
    """running handle_transpose_dims withs dims of Permute layer as identity permutation, expected no change"""
    # Arrange
    data_format = "channels_first"
    dims = [1, 2, 3, 4]
    permute_layer = Permute(dims)
    layer_config = permute_layer.get_config()

    # Action
    new_config = handle_transpose_dims(data_format, layer_config)

    # Assert
    assert new_config["dims"] == layer_config["dims"]


def test_transpose_dims_channels_first_transposable():
    # Arrange
    data_format = "channels_first"
    dims = [2, 1, 4, 3]
    permute_layer = Permute(dims)
    layer_config = permute_layer.get_config()

    # Action
    new_config = handle_transpose_dims(data_format, layer_config)

    # Assert
    assert new_config["dims"] == [4, 3, 2, 1]


def test_transpose_dims_channels_last_transposable_identity_permutation():
    """running handle_transpose_dims withs dims of Permute layer as identity permutation, expected no change"""
    # Arrange
    data_format = "channels_last"
    dims = [1, 2, 3, 4]
    permute_layer = Permute(dims)
    layer_config = permute_layer.get_config()

    # Action
    new_config = handle_transpose_dims(data_format, layer_config)

    # Assert
    assert new_config["dims"] == layer_config["dims"]


def test_transpose_dims_channels_last_transposable():
    # Arrange
    data_format = "channels_last"
    dims = [4, 3, 2, 1]
    permute_layer = Permute(dims)
    layer_config = permute_layer.get_config()

    # Action
    new_config = handle_transpose_dims(data_format, layer_config)

    # Assert
    assert new_config["dims"] == [2, 1, 4, 3]



from typing import Callable

import numpy as np
import pytest
import tensorflow as tf
from tensorflow.keras import backend as K

from keras_data_format_converter import convert_channels_first_to_last, convert_channels_last_to_first


def convert_and_assert(model_generator: Callable, data_format: str, test_infer: bool):
    if test_infer and not tf.test.gpu_device_name():
        pytest.skip("Skip! tensorflow supports the channels first format only on the GPU")
    # Arrange model and input sample
    K.set_image_data_format(data_format)
    model = model_generator()
    input_shape = model.input_shape
    input_shape = [1, *input_shape[1:]]
    input_np = np.random.uniform(0, 1, input_shape)
    transposed_inputs = [inp.name for inp in model.inputs]

    axes = list(range(len(input_shape)))
    # Action: Convert data_format of model
    if data_format == "channels_last":
        converted_model = convert_channels_last_to_first(model, transposed_inputs, verbose=False)
        axes = [axes[0], axes[-1], *axes[1:-1]]
        converted_input = np.transpose(input_np, axes)
    else:
        converted_model = convert_channels_first_to_last(model, transposed_inputs, verbose=False)
        axes = [axes[0], *axes[2:], axes[1]]
        converted_input = np.transpose(input_np, axes)

    # Since tensorflow Conv2D op currently only supports only the NHWC tensor format on the CPU
    if test_infer:
        converted_output = converted_model(converted_input)
        output = model(input_np)
        # Assert converted model is computing equally
        assert np.allclose(output, converted_output)



from setuptools import setup


def parse_requirements(filename):
    """ load requirements from a pip requirements file """
    lineiter = (line.strip() for line in open(filename))
    return [line for line in lineiter if line and not line.startswith("#")]


reqs = parse_requirements('requirements.txt')

with open('README.md') as f:
    long_description = f.read()

setup(name='keras_data_format_converter',
      packages=['keras_data_format_converter'],
      version="0.0.4rc2",
      description='Generates equal keras models with the desired data format',
      long_description=long_description,
      long_description_content_type='text/markdown',
      url='https://github.com/tensorleap/keras-data-format-converter',
      download_url='https://github.com/tensorleap/keras-data-format-converter/archive/v0.0.4rc2.tar.gz',
      author='Doron Har Noy',
      author_email='doron.harnoy@tensorleap.ai',
      classifiers=[
          'Development Status :: 3 - Alpha',
          'Intended Audience :: Science/Research',
          'License :: OSI Approved :: MIT License',
          'Operating System :: OS Independent',
          'Programming Language :: Python',
          'Topic :: Scientific/Engineering :: Image Recognition',
      ],
      keywords='machine-learning deep-learning keras neural-network tensorflow vgg resnet tensorleap',
      license='MIT',
      install_requires=reqs,
      zip_safe=False)



import pytest
from typing import Callable
from tensorflow.keras.applications import DenseNet121, EfficientNetB0, InceptionV3, InceptionResNetV2, \
    MobileNetV3Small, NASNetMobile, ResNet50V2, VGG16, Xception

from test.utils import convert_and_assert


@pytest.mark.slow
@pytest.mark.parametrize('data_format', ["channels_last", "channels_first"])
@pytest.mark.parametrize("test_infer", [True, False])
def test_densenet(data_format: str, test_infer: bool):
    convert_and_assert(DenseNet121, data_format, test_infer)

# Removing data_format = channels_first because of a bug in tensorflow (2.4) not able to generate EfficientNetB0 in
# that format
@pytest.mark.slow
@pytest.mark.parametrize('data_format', ["channels_last"])
@pytest.mark.parametrize("test_infer", [True, False])
def test_efficientnet(data_format: str, test_infer: bool):
    convert_and_assert(EfficientNetB0, data_format, test_infer)


@pytest.mark.slow
@pytest.mark.parametrize('model', [InceptionV3, InceptionResNetV2])
@pytest.mark.parametrize('data_format', ["channels_last", "channels_first"])
@pytest.mark.parametrize("test_infer", [True, False])
def test_inception(model: Callable, data_format: str, test_infer: bool):
    convert_and_assert(model, data_format, test_infer)


@pytest.mark.skip("fix issue with TFopLambda layer before running")
@pytest.mark.slow
@pytest.mark.parametrize('data_format', ["channels_last", "channels_first"])
@pytest.mark.parametrize("test_infer", [True, False])
def test_mobilenet(data_format: str, test_infer: bool):
    convert_and_assert(MobileNetV3Small, data_format, test_infer)


# NASNetMobile have a problem with creating a channels_first model
@pytest.mark.slow
@pytest.mark.parametrize('data_format', ["channels_last"])
@pytest.mark.parametrize("test_infer", [True, False])
def test_nasnet(data_format: str, test_infer: bool):
    convert_and_assert(NASNetMobile, data_format, test_infer)


@pytest.mark.slow
@pytest.mark.parametrize('data_format', ["channels_last", "channels_first"])
@pytest.mark.parametrize("test_infer", [True, False])
def test_resnet(data_format: str, test_infer: bool):
    convert_and_assert(ResNet50V2, data_format, test_infer)


@pytest.mark.slow
@pytest.mark.parametrize('data_format', ["channels_last", "channels_first"])
@pytest.mark.parametrize("test_infer", [True, False])
def test_vgg(data_format: str, test_infer: bool):
    convert_and_assert(VGG16, data_format, test_infer)


@pytest.mark.slow
@pytest.mark.parametrize('data_format', ["channels_last", "channels_first"])
@pytest.mark.parametrize("test_infer", [True, False])
def test_xception(data_format: str, test_infer: bool):
    convert_and_assert(Xception, data_format, test_infer)
